@@ -53,6 +53,9 @@ test_blob_empty (void)
   g_assert (hb_blob_is_immutable (hb_blob_get_empty ()));
   g_assert (hb_blob_get_empty () != NULL);
   g_assert (hb_blob_get_empty () == hb_blob_create (NULL, 0, HB_MEMORY_MODE_READONLY, NULL, NULL));
+  g_assert (hb_blob_get_empty () == hb_blob_create ("asdf", 0, HB_MEMORY_MODE_READONLY, NULL, NULL));
+  g_assert (hb_blob_get_empty () == hb_blob_create (NULL, -1, HB_MEMORY_MODE_READONLY, NULL, NULL));
+  g_assert (hb_blob_get_empty () == hb_blob_create ("asdfg", -1, HB_MEMORY_MODE_READONLY, NULL, NULL));
 
   blob = hb_blob_get_empty ();
   g_assert (blob == hb_blob_get_empty ());
@@ -113,7 +116,7 @@ free_up_free (fixture_t *fixture)
 static uintptr_t
 get_pagesize (void)
 {
-  uintptr_t pagesize = -1;
+  uintptr_t pagesize = (uintptr_t) -1;
 
 #if defined(HAVE_SYSCONF) && defined(_SC_PAGE_SIZE)
   pagesize = (uintptr_t) sysconf (_SC_PAGE_SIZE);
@@ -270,7 +273,7 @@ test_blob_subblob (fixture_t *fixture, gconstpointer user_data)
 
   if (mm == HB_MEMORY_MODE_DUPLICATE) {
     g_assert_cmpint (fixture->freed, ==, 1);
-    fixture->data = hb_blob_get_data (b, NULL);
+    fixture->data = (char *) hb_blob_get_data (b, NULL);
   } else {
     g_assert_cmpint (fixture->freed, ==, 0);
   }
